/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2001 Jason Baldridge, Gann Bierner, and Tom Morton
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////   
package opennlp.maxent;
import opennlp.maxent.io.*;
import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;
import java.io.*;

/**
 * An implementation of Generalized Iterative Scaling.  The reference paper
 * for this implementation was Adwait Ratnaparkhi's tech report at the
 * University of Pennsylvania's Institute for Research in Cognitive Science,
 * and is available at <a href ="ftp://ftp.cis.upenn.edu/pub/ircs/tr/97-08.ps.Z"><code>ftp://ftp.cis.upenn.edu/pub/ircs/tr/97-08.ps.Z</code></a>. 
 *
 * The slack parameter used in the above implementation has been removed by default
 * from the computation and a method for updating with Gaussian smoothing has been
 * added per Investigating GIS and Smoothing for Maximum Entropy Taggers, Clark and Curran (2002).  
 * <a href="http://acl.ldc.upenn.edu/E/E03/E03-1071.pdf"><code>http://acl.ldc.upenn.edu/E/E03/E03-1071.pdf</code></a>
 * The slack parameter can be used by setting <code>useSlackParameter</code> to true.
 * Gaussian smoothing can be used by setting <code>useGaussianSmoothing</code> to true. 
 * 
 * A prior can be used to train models which converge to the distribution which minimizes the
 * relative entropy between the distribution specified by the emperical constraints of the training
 * data and the specified prior.  By default, the uniform distribution is used as the prior.
 *    
 * @author Tom Morton
 * @author  Jason Baldridge
 * @version $Revision: 1.29 $, $Date: 2008/09/21 03:20:15 $
 */
class GISTrainer {

	private float[][]  predCount;
	/**
	 * Specifies whether unseen context/outcome pairs should be estimated as occur very infrequently.
	 */
	private boolean useSimpleSmoothing = false;
	/**
	 * Specifies whether a slack parameter should be used in the model.
	 */
	private boolean useSlackParameter = false;
	/** Specified whether parameter updates should prefer a distribution of parameters which
	 * is gaussian.
	 */
	private boolean useGaussianSmoothing = false;
	private double sigma = 2.0;

	// If we are using smoothing, this is used as the "number" of
	// times we want the trainer to imagine that it saw a feature that it
	// actually didn't see.  Defaulted to 0.1.
	private double _smoothingObservation = 0.1;
	private long end, begin;
	private double total_time;
	private boolean printMessages = false;

	/** Number of unique events which occured in the event set. */
	private int numUniqueEvents; 
	/** Number of predicates. */
	private int numPreds; 
	/** Number of outcomes. */
	private int numOutcomes; 
	private Evalable e;
	private String test_data;

	/** Records the array of predicates seen in each event. */
	private int[][] contexts;

	/** The value associated with each context. If null then context values are assumes to be 1. */
	private float[][] values;
	private float[][] column_value = null;
	private int[][] column_feature = null;
	/** List of outcomes for each event i, in context[i]. */
	private int[] outcomeList;

	/** Records the num of times an event has been seen for each event i, in context[i]. */
	private int[] numTimesEventsSeen;

	/** The number of times a predicate occured in the training data. */
	private int[] predicateCounts;

	private int cutoff;
	private boolean permutation = true;
	private boolean approx = false;
	/** Stores the String names of the outcomes.  The GIS only tracks outcomes
	  as ints, and so this array is needed to save the model to disk and
	  thereby allow users to know what the outcome was in human
	  understandable terms. */
	private String[] outcomeLabels;

	/** Stores the String names of the predicates. The GIS only tracks
	  predicates as ints, and so this array is needed to save the model to
	  disk and thereby allow users to know what the outcome was in human
	  understandable terms. */
	private String[] predLabels;

	/** Stores the observed expected values of the features based on training data. */
	private MutableContext[] observedExpects;

	/** Stores the estimated parameter value of each predicate during iteration */
	private MutableContext[] params;

	/** Stores the expected values of the features based on the current models */
	private MutableContext[] modelExpects;

	/** This is the prior distribution that the model uses for training. */
	private Prior prior;


	/** Observed expectation of correction feature. */
	private double cfObservedExpect;
	/** A global variable for the models expected value of the correction feature. */
	private double CFMOD;

	private final double NEAR_ZERO = 0.01;
	private final double LLThreshold = 0.0001;

	/** Stores the output of the current model on a single event durring
	 *  training.  This we be reset for every event for every itteration.  */
	double[] modelDistribution;
	/** Stores the number of features that get fired per event. */
	int[] numfeats;
	/** Initial probability for all outcomes. */

	private int solver_type;
	private boolean isvalue;
	EvalParameters evalParams;
	/**
	 * Creates a new <code>GISTrainer</code> instance which does
	 * not print progress messages about training to STDOUT.
	 *
	 */
	GISTrainer() {
		super();
	}

	/**
	 * Creates a new <code>GISTrainer</code> instance.
	 *
	 * @param printMessages sends progress messages about training to
	 *                      STDOUT when true; trains silently otherwise.
	 */
	GISTrainer(boolean printMessages) {
		this();
		this.printMessages = printMessages;
	}

	/**
	 * Sets whether this trainer will use smoothing while training the model.
	 * This can improve model accuracy, though training will potentially take
	 * longer and use more memory.  Model size will also be larger.
	 *
	 * @param smooth true if smoothing is desired, false if not
	 */
	public void setSmoothing(boolean smooth) {
		useSimpleSmoothing = smooth;
	}
	public void setGaussianSmoothing(double sigma_) {
		sigma = sigma_;
		useGaussianSmoothing = true;
	}
	public void Testing(MaxentModel model) {
		try {
			FileReader datafr = new FileReader(test_data);
			e.localEval(model, datafr, e, false);
		} catch (IOException e) {
		}
	}

	/**
	 * Sets whether this trainer will use smoothing while training the model.
	 * This can improve model accuracy, though training will potentially take
	 * longer and use more memory.  Model size will also be larger.
	 *
	 * @param timesSeen the "number" of times we want the trainer to imagine
	 *                  it saw a feature that it actually didn't see
	 */
	public void setSmoothingObservation(double timesSeen) {
		_smoothingObservation = timesSeen;
	}


	/**
	 * Trains a GIS model on the event in the specified event stream, using the specified number
	 * of iterations and the specified count cutoff.
	 * @param eventStream A stream of all events.
	 * @param iterations The number of iterations to use for GIS.
	 * @param cutoff The number of times a feature must occur to be included.
	 * @return A GIS model trained with specified 
	 */
	public GISModel trainModel(EventStream eventStream, int iterations, int cutoff) {
		return trainModel(iterations, new OnePassDataIndexer(eventStream,cutoff),cutoff);
	}
	public void setTesting(String test_data, Evalable e) {
		this.test_data = test_data;
		this.e = e;
	}

	public void setSolverType(int solver, boolean isvalue) {
		System.out.println("solver_type " + solver);
		this.solver_type = solver;
		this.isvalue = isvalue;
	}

	/**
	 * Train a model using the GIS algorithm.
	 *
	 * @param iterations  The number of GIS iterations to perform.
	 * @param di The data indexer used to compress events in memory.
	 * @return The newly trained model, which can be used immediately or saved
	 *         to disk using an opennlp.maxent.io.GISModelWriter object.
	 */
	public GISModel trainModel(int iterations, DataIndexer di, int cutoff) {
		return trainModel(iterations,di,new UniformPrior(),cutoff);
	}

	/**
	 * Train a model using the GIS algorithm.
	 *
	 * @param iterations  The number of GIS iterations to perform.
	 * @param di The data indexer used to compress events in memory.
	 * @param modelPrior The prior distribution used to train this model.
	 * @return The newly trained model, which can be used immediately or saved
	 *         to disk using an opennlp.maxent.io.GISModelWriter object.
	 */
	public GISModel trainModel(int iterations, DataIndexer di, Prior modelPrior, int cutoff) {
		/************** Incorporate all of the needed info ******************/
		display("Incorporating indexed data for training...  \n");
		contexts = di.getContexts();
		values = di.getValues();
		this.cutoff = cutoff;
		predicateCounts = di.getPredCounts();
		numTimesEventsSeen = di.getNumTimesEventsSeen();
		numUniqueEvents = contexts.length;
		this.prior = modelPrior;
		//printTable(contexts);
		outcomeLabels = di.getOutcomeLabels();
		outcomeList = di.getOutcomeList();
		numOutcomes = outcomeLabels.length;

		predLabels = di.getPredLabels();
		prior.setLabels(outcomeLabels, predLabels);
		numPreds = predLabels.length;
		int[] last = new int[numPreds];

		for (int i = 0; i < numPreds; i++)
			last[i] = -1;
		// kill duplicate features
		for (int i = 0; i < numUniqueEvents; i++) {
			int nowlength = 0;
			int[] tmpcontext = new int[contexts[i].length];
			for (int j = 0; j < contexts[i].length; j++)
				if (last[contexts[i][j]] != i) {
					tmpcontext[nowlength++] = contexts[i][j];
					last[contexts[i][j]] = i;
				}
			contexts[i] = new int[nowlength];
			for (int j = 0; j < nowlength; j++)
				contexts[i][j] = tmpcontext[j];
		}

		if (isvalue) {
			values = new float[contexts.length][];
			for (int i = 0; i < contexts.length; i++) {
				values[i] = new float[contexts[i].length];
				for (int j = 0; j < contexts[i].length; j++)
					values[i][j] = (float) 1.0;
			}
		}

		// determine the correction constant and its inverse
		int correctionConstant = 1;
		for (int ci = 0; ci < contexts.length; ci++) {
			if (values == null || values[ci] == null) {
				if (contexts[ci].length > correctionConstant) {
					correctionConstant = contexts[ci].length;
				}
			}
			else {
				float cl = values[ci][0];
				for (int vi=1;vi<values[ci].length;vi++) {
					cl+=values[ci][vi];
				}

				if (cl > correctionConstant) {
					correctionConstant=(int) Math.ceil(cl);
				}
			}
		}
		display("done.\n");


		display("\tNumber of Event Tokens: " + numUniqueEvents + "\n");
		display("\t    Number of Outcomes: " + numOutcomes + "\n");
		display("\t  Number of Predicates: " + numPreds + "\n");

		// set up feature arrays
		predCount = new float[numPreds][numOutcomes];
		for (int ti = 0; ti < numUniqueEvents; ti++) {
			for (int j = 0; j < contexts[ti].length; j++) {
				if (values != null && values[ti] != null) {
					predCount[contexts[ti][j]][outcomeList[ti]] += numTimesEventsSeen[ti]*values[ti][j];
				}
				else {          
					predCount[contexts[ti][j]][outcomeList[ti]] += numTimesEventsSeen[ti];
				}
			}
		}

		//printTable(predCount);
		di = null; // don't need it anymore

		// A fake "observation" to cover features which are not detected in
		// the data.  The default is to assume that we observed "1/10th" of a
		// feature during training.
		final double smoothingObservation = _smoothingObservation;

		// Get the observed expectations of the features. Strictly speaking,
		// we should divide the counts by the number of Tokens, but because of
		// the way the model's expectations are approximated in the
		// implementation, this is cancelled out when we compute the next
		// iteration of a parameter, making the extra divisions wasteful.
		params = new MutableContext[numPreds];
		modelExpects = new MutableContext[numPreds];
		observedExpects = new MutableContext[numPreds];
		evalParams = new EvalParameters(params,0,correctionConstant,numOutcomes);
		int[] activeOutcomes = new int[numOutcomes];
		int[] outcomePattern;
		int[] allOutcomesPattern= new int[numOutcomes];
		for (int oi = 0; oi < numOutcomes; oi++) {
			allOutcomesPattern[oi] = oi;
		}
		int numActiveOutcomes = 0;
		for (int pi = 0; pi < numPreds; pi++) {
			numActiveOutcomes = 0;
			if (useSimpleSmoothing) {
				numActiveOutcomes = numOutcomes;
				outcomePattern = allOutcomesPattern;
			}
			else { //determine active outcomes
				for (int oi = 0; oi < numOutcomes; oi++) {
					if (predCount[pi][oi] > 0 && predicateCounts[pi] >= cutoff) {
						activeOutcomes[numActiveOutcomes] = oi;
						numActiveOutcomes++;
					}
				}
				if (numActiveOutcomes == numOutcomes) {
					outcomePattern = allOutcomesPattern;
				}
				else {
					outcomePattern = new int[numActiveOutcomes];
					for (int aoi=0;aoi<numActiveOutcomes;aoi++) {
						outcomePattern[aoi] = activeOutcomes[aoi];
					}
				}
			}
			params[pi] = new MutableContext(outcomePattern,new double[numActiveOutcomes]);
			modelExpects[pi] = new MutableContext(outcomePattern,new double[numActiveOutcomes]);
			observedExpects[pi] = new MutableContext(outcomePattern,new double[numActiveOutcomes]);
			for (int aoi=0;aoi<numActiveOutcomes;aoi++) {
				int oi = outcomePattern[aoi];
				params[pi].setParameter(aoi, 0.0);
				modelExpects[pi].setParameter(aoi, 0.0);
				if (predCount[pi][oi] > 0) {
					observedExpects[pi].setParameter(aoi, predCount[pi][oi]);
				}
				else if (useSimpleSmoothing) { 
					observedExpects[pi].setParameter(aoi,smoothingObservation);
				}
			}
		}

		// compute the expected value of correction
		if (useSlackParameter) {
			int cfvalSum = 0;
			for (int ti = 0; ti < numUniqueEvents; ti++) {
				for (int j = 0; j < contexts[ti].length; j++) {
					int pi = contexts[ti][j];
					if (!modelExpects[pi].contains(outcomeList[ti])) {
						cfvalSum += numTimesEventsSeen[ti];
					}
				}
				cfvalSum += (correctionConstant - contexts[ti].length) * numTimesEventsSeen[ti];
			}
			if (cfvalSum == 0) {
				cfObservedExpect = Math.log(NEAR_ZERO); //nearly zero so log is defined
			}
			else {
				cfObservedExpect = Math.log(cfvalSum);
			}
		}
		// predCount = null; // don't need it anymore

		display("...done.\n");

		modelDistribution = new double[numOutcomes];
		numfeats = new int[numOutcomes];

		/***************** Find the parameters ************************/
		display("Computing model parameters...\n");
		System.out.println("useGaussianSmoothing = " + useGaussianSmoothing + " sigma= " + sigma);
		if (solver_type == 0)
			findParameters(iterations);
		else if (solver_type == 1) {
			if (useGaussianSmoothing) {
				if (isvalue)
#CALL sequential_SCGIS_regular_isvalue(iterations);
				else
#CALL sequential_SCGIS_regular(iterations);
			} else {
				if (isvalue)
#CALL sequential_SCGIS_isvalue(iterations);
				else
#CALL sequential_SCGIS(iterations);
			}
		} else if (solver_type == 2) {
			if (useGaussianSmoothing) {
				if (isvalue)
#CALL sequential_cd_noapprox_regular_isvalue(iterations);
				else
#CALL sequential_cd_noapprox_regular(iterations);
			} else {
				if (isvalue)
#CALL sequential_cd_noapprox_isvalue(iterations);
				else
#CALL sequential_cd_noapprox(iterations);
			}
		} else if (solver_type == 3) {
			if (useGaussianSmoothing) {
				if (isvalue)
#CALL sequential_cd_regular_isvalue(iterations);
				else
#CALL sequential_cd_regular(iterations);
			} else {
				if (isvalue)
#CALL sequential_cd_isvalue(iterations);
				else
#CALL sequential_cd(iterations);
			}
		} else if (solver_type == 4) {
			if (useGaussianSmoothing) {
				if (isvalue)
#CALL sequential_cd_regular_nocheckfirst_noapprox_isvalue(iterations);
				else
#CALL sequential_cd_regular_noapprox_nocheckfirst(iterations);
			} else {
				if (isvalue)
#CALL sequential_cd_noapprox_nocheckfirst_isvalue(iterations);
				else
#CALL sequential_cd_noapprox_nocheckfirst(iterations);

			}
		} else if (solver_type == 5) {
			lbfgs(iterations);
		} else if (solver_type == 6) {
			tron(iterations);
		} else if (solver_type == 7) {
			cddual(iterations);
		} else if (solver_type == 8) {
			EG(iterations);
		} 
		else {
			System.out.println("SOLVER " + solver_type + "unknown");
			return null;
		}
		System.exit(1);
		/*************** Create and return the model ******************/
		return new GISModel(params, predLabels, outcomeLabels, correctionConstant, evalParams.correctionParam);
	}

	private void Transform(int[][] row_feature, float[][] row_value, int n, int cutoff) {
		int i, x, ci, pi;
		column_feature = new int[n][];
		int [] count = new int[n];
		long begin = System.currentTimeMillis();
		System.out.println("Transforming to columnn format");
		if (row_value == null)
			column_value = null;
		else
			column_value = new float[n][];
		for (i = 0; i < n; i++)
			count[i] = 0;
		for (x = 0; x < contexts.length; x++)
			for (ci = 0; ci < contexts[x].length; ci++) {
				pi = contexts[x][ci];
				if (predicateCounts[pi] >= cutoff)
					count[pi]++;
			}

		for (i = 0; i < n; i++) {
			column_feature[i] = new int[count[i]];
			if (row_value != null)
				column_value[i] = new float[count[i]];
			count[i] = 0;
		}

		for (x = 0; x < contexts.length; x++)
			for (ci = 0; ci < contexts[x].length; ci++) {
				pi = contexts[x][ci];
				if (predicateCounts[pi] >= cutoff) {
					column_feature[pi][count[pi]++] = x;
					if (values != null)
						column_value[pi][count[pi] - 1] = row_value[x][ci];
				}
			}
		long end = System.currentTimeMillis();	
	}

	private void recompute(int[][] row_feature, float[][] row_value, double[][] expSwf, double[] SF) {
		int i, j, k;
		float[] nowvalue = null;
		for (i = 0; i < expSwf.length; i++)
			for (j = 0; j < expSwf[i].length; j++)
				expSwf[i][j] = 1;

		for (i = 0; i < row_feature.length; i++) {
			double[] parameters = params[i].getParameters();
			int[] nowfeature = row_feature[i];
			if (nowvalue != null)
				nowvalue = row_value[i];
			int[] activeOutcomes = observedExpects[i].getOutcomes();
			for (int aoi = 0; aoi < parameters.length; aoi++) {
				int oi = activeOutcomes[aoi];
				for (j = 0; j < nowfeature.length; j++) {
					if (nowvalue != null)
						expSwf[oi][nowfeature[j]] *= Math.exp(parameters[aoi] * nowvalue[aoi]);
					else
						expSwf[oi][nowfeature[j]] *= Math.exp(parameters[aoi]);
				}
			}
		}	
		for (i = 0; i < SF.length; i++) {
			SF[i] = 0;
			for (j = 0; j < expSwf.length; j++)
				SF[i] += expSwf[j][i];
		}
	}


#FUNCTION sequential
	private void sequential(int iterations) {
		int iter = 1, yTag;
		int xSize = numUniqueEvents, ySize = numOutcomes, n = numPreds;
		double[] ptildeX = new double[xSize];
		int[] indexf = new int[n];
		double[] SptildeX = new double[numPreds];
		double [] expSwf_yTag; // a pointer point to expSwf[yTag]
		double sigma2r = 1.0 / sigma, expd, expd_1, Cond, SPCond = 0, SPCond_2 = 0, D, d, SPCond_3 = 0;
		double eta = 0.001;
		int x, y, i, j, t;
		Random rgenerator = new Random(10);
		double Dp, Dpp, cond, cond1;
		double time_begin, time_end, time_pass = 0;
		double [] SF = new double[xSize];
		double [][] expSwf = new double[ySize][xSize];

		S_store = new double[numUniqueEvents][numOutcomes];
		Transform(contexts, values, n, cutoff);
		System.out.println("cutoff = " + cutoff);
		begin = System.currentTimeMillis();
		for (x = 0; x < xSize; x++) {
			SF[x] = ySize;
		}
		for (y = 0; y < ySize; y++)
			for (x = 0; x < xSize; x++)
				expSwf[y][x] = 1.0;
		for (t = 0; t < n; t++) {
			indexf[t] = t;
			SptildeX[t] = 0;
			for (i = 0; i < column_feature[t].length; i++)
				SptildeX[t] +=  numTimesEventsSeen[column_feature[t][i]];
		}

		System.out.println("strarting loop");
		while (iter <= iterations) {
			// one outer iteration
			time_begin = System.currentTimeMillis();
			int count_d = 0;
			for (t = 0; t < n; t++) {		  
				double [] ftildeArr = observedExpects[t].getParameters();
				int[] nowfeature = column_feature[t];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				double[] nowparameters = params[t].getParameters();
				float[] nowvalue = null;
#IFDEF isvalue
				nowvalue = column_value[t];
#END isvalue
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					int oi = activeOutcomes[aoi];
					expSwf_yTag  = expSwf[oi];
#IFDEF SCGIS
					SPCond = 0;
					SPCond_2 = 0;
#IFDEF isvalue
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						SPCond += numTimesEventsSeen[x] * expSwf_yTag[x] / SF[x] * nowvalue[j];
					}
#ELSE isvalue
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						Cond = expSwf_yTag[x] / SF[x];
						SPCond += numTimesEventsSeen[x] * Cond;
					}
#END isvalue
#IFDEF regular
					d = 0;
					do {
						Dp = -ftildeArr[aoi] + SPCond * Math.exp(d) + sigma2r * (d + nowparameters[aoi]);
						Dpp = SPCond * Math.exp(d) + sigma2r;
						d -= (Dp / Dpp);
					} while (Math.abs(Dp) > 1e-5);
#ELSE regular
					d = Math.log(ftildeArr[aoi] / SPCond);
#END regular
#ELSE SCGIS
#IFDEF cd
					// Calcuate Cond
					SPCond = 0;
					SPCond_2 = 0;
					SPCond_3 = 0;
#IFDEF isvalue
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						Cond = expSwf_yTag[x] / SF[x];
						SPCond += numTimesEventsSeen[x] * Cond * nowvalue[j];
						SPCond_3 += numTimesEventsSeen[x] * Cond * nowvalue[j] * nowvalue[j];
						SPCond_2 += numTimesEventsSeen[x] * Cond * Cond * nowvalue[j] * nowvalue[j];
					}
#ELSE isvalue
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						Cond = expSwf_yTag[x] / SF[x];
						SPCond += numTimesEventsSeen[x] * Cond;
						SPCond_2 += numTimesEventsSeen[x] * Cond * Cond;
					}
#END isvalue
					//Dp,  Dpp, D
					Dp = (SPCond - ftildeArr[aoi]); //scaling
#IFDEF isvalue
					Dpp = (SPCond_3 - SPCond_2);
#ELSE isvalue
					Dpp = (SPCond - SPCond_2);
#END isvalue
#IFDEF regular
					Dp += nowparameters[aoi] * sigma2r;
					Dpp += sigma2r;
#END regular
					if (Math.abs(Dp) < 1e-12)
						continue;

					// Too huge d may incur numerical error in exponential operation
					d = Math.max(Math.min(-Dp / Dpp , 5), -5);
					count_d = 0;
					while (true) {
						expd = Math.exp(d);
						expd_1 = expd - 1;
#IFDEF nocheckfirst
#ELSE nocheckfirst
						cond1 = Math.log(1 + SPCond * expd_1 / SptildeX[t]) * SptildeX[t] - d * ftildeArr[aoi] - eta * d * Dp;
#IFDEF regular
						cond1 += 0.5 * d * d * sigma2r + nowparameters[aoi] * d * sigma2r;
#END regular
						if (cond1 <= 0.0)
							break;
#END nocheckfirst
#IFDEF noapprox
						cond = 0;
						for (j = 0; j < nowfeature.length; j++) {
							x = nowfeature[j];
							cond += numTimesEventsSeen[x] * Math.log(1 + expSwf_yTag[x] * expd_1 / SF[x]);
						}			
						count_d++;
						cond -= (d * ftildeArr[aoi] + eta * d * Dp);
#IFDEF regular
						cond += (0.5 * d + nowparameters[aoi]) * d * sigma2r;
#END regular
						if (count_d > 50) {
							recompute(column_feature, column_value, expSwf, SF);
							System.out.format("warning: line search is too deep, recompute expSwf");
						}

						if (cond <= 0.0) {
							break;
						}
#END noapprox
						d *= 0.5;
					}
#END cd
#END SCGIS
					expd = Math.exp(d);
					expd_1 = expd - 1;
					params[t].updateParameter(aoi, d);
#IFDEF isvalue
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						expd = Math.exp(d * nowvalue[j]);
						SF[x] += expSwf_yTag[x] * (expd - 1);
						expSwf_yTag[x] *= expd;
					}
#ELSE isvalue
					//update expSwf, SF
					for (j = 0; j < nowfeature.length; j++) {
						x = nowfeature[j];
						double tmp = SF[x];
						SF[x] += expSwf_yTag[x] * expd_1;
						expSwf_yTag[x] *= expd;
					}
#END isvalue
				}
			}
			time_end = System.currentTimeMillis();
			time_pass += time_end - time_begin;
			double Dt = 0;
#IFDEF regular
			for (t = 0; t < n; t++) {
				double[] parameters = params[t].getParameters();
				for (int aoi = 0; aoi < parameters.length; aoi++)
					Dt += parameters[aoi] * parameters[aoi];
			}
			Dt = Dt * sigma2r * 0.5;
#END regular
			for (x = 0; x < xSize; x++) {
				Dt -= Math.log(expSwf[outcomeList[x]][x] / SF[x]) * numTimesEventsSeen[x];
			}

			double grad_norm = 0;	  
			double[][] grad = new double[n][];
			double[][] parameter = new double[n][];
			double w_norm = 0;
			for (t = 0; t < n; t++) {
				parameter[t] = params[t].getParameters();
				grad[t] = new double[parameter[t].length];
				for(int aoi = 0; aoi < grad[t].length; aoi++)
					w_norm += parameter[t][aoi]*parameter[t][aoi];
			}
			double vv = valueAt(parameter);
			derivativeAt_store(parameter, grad);
			for (t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < grad[t].length; aoi++) {
					grad_norm += grad[t][aoi] * grad[t][aoi]; 
				}
			}
			grad_norm = Math.sqrt(grad_norm);

			double grad_norm2 = 0;
			derivativeAtRofu(parameter, grad);
			for (t = 0; t < n; t++) {
				double[] gradt = grad[t];
				for (int aoi = 0; aoi < grad[t].length; aoi++) {
					grad_norm2 += gradt[aoi] * gradt[aoi]; 
				}
			}
			grad_norm2 = Math.sqrt(grad_norm2);

			System.out.format(" iter %d time %.4f f %.12g g %.12g ", iter , (time_pass) / 1000.0, Dt, grad_norm);

			MaxentModel m = new GISModel(params, predLabels, outcomeLabels, 1, 0);
			Testing(m);
			iter++;
		}

		// kill a bunch of these big objects now that we don't need them
		observedExpects = null;
		modelExpects = null;
		numTimesEventsSeen = null;
		contexts = null;
	}
#ENDFUNCTION sequential

	/* Estimate and return the model parameters. */
	private void findParameters(int iterations) {
		double prevLL = 0.0;
		double currLL = 0.0;	
		display("Performing " + iterations + " iterations.\n");
		total_time = 0.0;
		for (int i = 1; i <= iterations; i++) {
			currLL = nextIteration();
			double[][] grad = new double[numPreds][];
			double[][] parameter = new double[numPreds][];
			double grad_norm = 0.0;
			for (int t = 0; t < numPreds; t++) {
				parameter[t] = params[t].getParameters();
				grad[t] = new double[parameter[t].length];
			}

			derivativeAt(parameter, grad);
			for (int t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < grad[t].length; aoi++) {
					grad_norm += grad[t][aoi] * grad[t][aoi];
				}
			}
			grad_norm = Math.sqrt(grad_norm);

			System.out.format(" iter %d time %.4f f %.12f g %.12f ", i, total_time, currLL, grad_norm);
			MaxentModel m = new GISModel(params, predLabels, outcomeLabels, 1, 0);
			Testing(m);
			prevLL = currLL;
		}

		// kill a bunch of these big objects now that we don't need them
		observedExpects = null;
		modelExpects = null;
		numTimesEventsSeen = null;
		contexts = null;
	}

	//modeled on implementation in  Zhang Le's maxent kit
	private double gaussianUpdate(int predicate, int oid, int n, double correctionConstant) {
		double param = params[predicate].getParameters()[oid];
		double x = 0.0;
		double x0 = 0.0;
		double f;
		double tmp;
		double fp;
		double modelValue = modelExpects[predicate].getParameters()[oid];
		double observedValue = observedExpects[predicate].getParameters()[oid];
		for (int i = 0; i < 50; i++) {
			tmp = modelValue * Math.exp(correctionConstant * x0);
			f = tmp + (param + x0) / sigma - observedValue;
			fp = tmp * correctionConstant + 1 / sigma;
			if (fp == 0) {
				break;
			}
			x = x0 - f / fp;
			if (x > 40 / correctionConstant)
				x = 40 / correctionConstant;
			if (Math.abs(x - x0) < 0.000001) {
				x0 = x;
				break;
			}
			x0 = x;
		}
		return x0;
	}

	/* Compute one iteration of GIS and retutn log-likelihood.*/
	private double nextIteration() {
		// compute contribution of p(a|b_i) for each feature and the new
		// correction parameter
		double loglikelihood = 0.0;
		CFMOD = 0.0;
		int numEvents = 0;
		int numCorrect = 0;
		begin = System.currentTimeMillis();
		/// compute exp[w_i]
		double[][] exp_tmp = new double[numPreds][];
		for (int i = 0; i < numPreds; i++) {
			double[] activeParameters = params[i].getParameters();
			exp_tmp[i] = new double[activeParameters.length];
			for (int j = 0; j < activeParameters.length; j++)
				exp_tmp[i][j] = Math.exp(activeParameters[j]);
		}

		for (int ei = 0; ei < numUniqueEvents; ei++) {
			for (int j = 0; j < numOutcomes; j++)
				modelDistribution[j] = 1;

			for (int j = 0; j < contexts[ei].length; j++) {
				int t = contexts[ei][j];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
					modelDistribution[activeOutcomes[aoi]] *= exp_tmp[t][aoi];
			}
			double tmp = 0.0;
			for (int j = 0; j < numOutcomes; j++)
				tmp += modelDistribution[j];

			for (int j = 0; j < numOutcomes; j++)
				modelDistribution[j] /= tmp;

			if (isvalue) {
				for (int j = 0; j < contexts[ei].length; j++) {
					int pi = contexts[ei][j];
					if (predicateCounts[pi] >= cutoff) {
						int[] activeOutcomes = modelExpects[pi].getOutcomes();
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
							int oi = activeOutcomes[aoi];
							modelExpects[pi].updateParameter(aoi, modelDistribution[oi] * values[ei][j] * numTimesEventsSeen[ei]);
						}
					}
				}
			}
			else {
				for (int j = 0; j < contexts[ei].length; j++) {
					int pi = contexts[ei][j];
					if (predicateCounts[pi] >= cutoff) {
						int[] activeOutcomes = modelExpects[pi].getOutcomes();
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
							int oi = activeOutcomes[aoi];
							modelExpects[pi].updateParameter(aoi, modelDistribution[oi] * numTimesEventsSeen[ei]);
						}
					}
				}
			}

			loglikelihood += Math.log(modelDistribution[outcomeList[ei]]) * numTimesEventsSeen[ei];
			numEvents += numTimesEventsSeen[ei];
			if (printMessages) {
				int max = 0;
				for (int oi = 1; oi < numOutcomes; oi++) {
					if (modelDistribution[oi] > modelDistribution[max]) {
						max = oi;
					}
				}
				if (max == outcomeList[ei]) {
					numCorrect += numTimesEventsSeen[ei];
				}
			}

		}
		end = System.currentTimeMillis();
		total_time += (end - begin) / 1000.0;

		double tmp = 0;
		if (useGaussianSmoothing) {
			for (int pi = 0; pi < numPreds; pi++) {
				double[] parameters = params[pi].getParameters();
				for (int aoi = 0; aoi < parameters.length; aoi++) {
					loglikelihood -= parameters[aoi] * parameters[aoi] / sigma / 2/*/evalParams.correctionConstant*/;
				}
			}
		}
		begin = System.currentTimeMillis();

		// compute the new parameter values
		if (useGaussianSmoothing) {
			for (int pi = 0; pi < numPreds; pi++) {
				double[] observed = observedExpects[pi].getParameters();
				double[] model = modelExpects[pi].getParameters();
				int[] activeOutcomes = params[pi].getOutcomes();
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					params[pi].updateParameter(aoi,gaussianUpdate(pi,aoi,numEvents,evalParams.correctionConstant));
					modelExpects[pi].setParameter(aoi, 0.0); // re-initialize to 0.0's
				}
			}
		} 
		else {
			for (int pi = 0; pi < numPreds; pi++) {
				double[] observed = observedExpects[pi].getParameters();
				double[] model = modelExpects[pi].getParameters();
				int[] activeOutcomes = params[pi].getOutcomes();
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					if (model[aoi] == 0) {
						System.err.println("Model expects == 0 for "+predLabels[pi]+" "+outcomeLabels[aoi]);
					}
					params[pi].updateParameter(aoi,(Math.log(observed[aoi]) - Math.log(model[aoi])));
					modelExpects[pi].setParameter(aoi, 0.0); // re-initialize to 0.0's
				}
			}

		}

		if (CFMOD > 0.0 && useSlackParameter)
			evalParams.correctionParam += (cfObservedExpect - Math.log(CFMOD));
		//display(". loglikelihood=" + loglikelihood + "\t" + ((double) numCorrect / numEvents) + "\n");
		end = System.currentTimeMillis();
		total_time += (end - begin) / 1000.0;
		return -1.0*(loglikelihood);
	}

	private void display(String s) {
		if (printMessages)
			System.out.print(s);
	}


	private void clearStuff() {
		sList = null;
		yList = null;
		roList = null;
	}


	// computes d = a + b * c
	private static double[][] plusAndConstMult(double[][] a, double[][] b, double c, double[][] d) {
		for (int i = 0; i < a.length; i++)
			for (int j = 0; j < a[i].length; j++)
				d[i][j] = a[i][j] + c * b[i][j];
		return d;
	}

	private static double inner_product(double[][] a, double[][] b) {
		double result = 0.0;
		for (int i = 0; i < a.length; i++)
			for (int j = 0; j < a[i].length; j++)
				result += a[i][j] * b[i][j];
		return result;
	}

	private List<double[][]> sList;// = new ArrayList<double[][]>();
	private List<double[][]> yList;// = new ArrayList<double[][]>();
	private List<Double> roList;//= new ArrayList<Double>();

	private void computeDir(double[][] dir, double[][] fg) { /* throws SurpriseConvergence*/
		for (int i = 0; i < fg.length; i++)
			for (int  j = 0; j < fg[i].length; j++)
				dir[i][j] = fg[i][j];	

		int sList_size = sList.size();
		double[] as = new double[sList_size];

		for (int i = sList_size - 1; i >= 0; i--) {
			as[i] = roList.get(i) * inner_product(sList.get(i), dir);
			plusAndConstMult(dir, yList.get(i), -as[i], dir);
		}

		// multiply by hessian approximation
		if (sList_size != 0) {
			double[][] y = yList.get(sList_size - 1);
			double yDotY = inner_product(y, y);
			double gamma = inner_product(sList.get(sList_size - 1), y) / yDotY;
			for (int i = 0; i < dir.length; i++)
				for (int j = 0; j < dir[i].length; j++)
					dir[i][j] *= gamma;
		}


		for (int i = 0; i < sList_size; i++) {
			double b = roList.get(i) * inner_product(yList.get(i), dir);
			plusAndConstMult(dir, sList.get(i), as[i] - b, dir);
		}

		for (int i = 0; i < dir.length; i++)
			for (int j = 0; j < dir[i].length; j++)
				dir[i][j] *= (-1);
		double len = 0.0;
		for (int i = 0; i < dir.length; i++)
			for (int j = 0; j < dir[i].length; j++)
				len += Math.abs(dir[i][j]);
	}

	private void derivativeAtRofu(double[][] x, double[][] grad) {
		for (int i = 0; i < grad.length; i++)
			for (int j = 0; j < grad[i].length; j++)
				grad[i][j] = 0;
		if (useGaussianSmoothing) {
			for (int t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < x[t].length; aoi++) {
					grad[t][aoi] += x[t][aoi] / sigma;
				}
			}
		}

		double[] wTf = new double[numOutcomes];
		for (int i = 0; i < numUniqueEvents; i++) {
			for (int j = 0; j < numOutcomes; j++)
				wTf[j] = 0;

			if (isvalue) {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] += x[t][aoi] * values[i][j];
				}
			} else {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] += x[t][aoi];
				}

			}

			for (int j = 0; j < numOutcomes; j++) {
				// Too huge wTf may incur numerical error in exponential operation
				//wTf[j] = Math.min(20, Math.max(-20,wTf[j])); 
				wTf[j] = Math.exp(wTf[j]);
			}

			double tmp = 0.0;
			for (int j = 0; j < numOutcomes; j++)
				tmp += wTf[j];
			for (int j = 0; j < numOutcomes; j++)
				wTf[j] = wTf[j] / tmp;

			if (isvalue) {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += wTf[activeOutcomes[aoi]] * numTimesEventsSeen[i] * values[i][j];
				}
			} else {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += wTf[activeOutcomes[aoi]] * numTimesEventsSeen[i];
				}

			}
		}

		for (int i = 0; i < grad.length; i++) {
			double[] parameters = observedExpects[i].getParameters();
			for (int j = 0;  j < grad[i].length; j++)
				grad[i][j] -= parameters[j];
		}
	}
	private void derivativeAt(double[][] x, double[][] grad) {
		for (int i = 0; i < grad.length; i++)
			for (int j = 0; j < grad[i].length; j++)
				grad[i][j] = 0;
		if (useGaussianSmoothing) {
			for (int t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < x[t].length; aoi++) {
					grad[t][aoi] += x[t][aoi] / sigma;
				}
			}
		}

		double[] wTf = new double[numOutcomes];
		for (int i = 0; i < numUniqueEvents; i++) {
			for (int j = 0; j < numOutcomes; j++)
				wTf[j] = 0;

			if (isvalue) {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] += x[t][aoi] * values[i][j];
				}
			} else {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] += x[t][aoi];
				}

			}

			for (int j = 0; j < numOutcomes; j++) {
				// Too huge wTf may incur numerical error in exponential operation
				wTf[j] = Math.min(20, Math.max(-20,wTf[j])); 
				wTf[j] = Math.exp(wTf[j]);
			}

			double tmp = 0.0;
			for (int j = 0; j < numOutcomes; j++)
				tmp += wTf[j];
			for (int j = 0; j < numOutcomes; j++)
				wTf[j] = wTf[j] / tmp;

			if (isvalue) {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += wTf[activeOutcomes[aoi]] * numTimesEventsSeen[i] * values[i][j];
				}
			} else {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += wTf[activeOutcomes[aoi]] * numTimesEventsSeen[i];
				}

			}
		}

		for (int i = 0; i < grad.length; i++) {
			double[] parameters = observedExpects[i].getParameters();
			for (int j = 0;  j < grad[i].length; j++)
				grad[i][j] -= parameters[j];
		}
	}

	double[][] S_store;
	double[] T_store;
	private void derivativeAt_store(double[][] x, double[][] grad) {
		for (int i = 0; i < grad.length; i++)
			for (int j = 0; j < grad[i].length; j++)
				grad[i][j] = 0;
		if (useGaussianSmoothing) {
			for (int t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < x[t].length; aoi++) {
					grad[t][aoi] += x[t][aoi] / sigma;
				}
			}
		}

		for (int i = 0; i < numUniqueEvents; i++) {
			if (isvalue) {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += S_store[i][activeOutcomes[aoi]] * numTimesEventsSeen[i] * values[i][j];
				}
			} else {
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						grad[t][aoi] += S_store[i][activeOutcomes[aoi]] * numTimesEventsSeen[i];
				}

			}
		}

		for (int i = 0; i < grad.length; i++) {
			double[] parameters = observedExpects[i].getParameters();
			for (int j = 0;  j < grad[i].length; j++)
				grad[i][j] -= parameters[j];
		}
	}


	private double valueAt(double[][] x) {
		double Dt = 0;
		if (useGaussianSmoothing) {
			for (int t = 0; t < numPreds; t++) {
				for (int aoi = 0; aoi < x[t].length; aoi++) {
					Dt += x[t][aoi] * x[t][aoi];
				}
			}
			Dt = Dt * 0.5 / sigma;
		}

		double[] wTf = new double[numOutcomes];
		if (isvalue) {
			for (int i = 0; i < numUniqueEvents; i++) {
				for (int j = 0; j < numOutcomes; j++)
					wTf[j] = 0;
				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] += x[t][aoi] * values[i][j];
				}
				for (int j = 0; j < numOutcomes; j++) {
					wTf[j] = Math.exp(wTf[j]);
				}
				double tmp = 0.0;
				for (int j = 0; j < numOutcomes; j++)
					tmp += wTf[j];

				if (!Double.isInfinite(wTf[outcomeList[i]])) {
					Dt -= Math.log(wTf[outcomeList[i]] / tmp) * numTimesEventsSeen[i];
				}
			}
		} else {
			double[][] tmp_exp = new double[x.length][];
			for (int i = 0; i < x.length; i++) {
				tmp_exp[i] = new double[x[i].length];
				for (int j = 0; j < tmp_exp[i].length; j++)
					tmp_exp[i][j] = Math.exp(x[i][j]);
			}

			for (int i = 0; i < numUniqueEvents; i++) {
				for (int j = 0; j < numOutcomes; j++)
					wTf[j] = 1;

				for (int j = 0; j < contexts[i].length; j++) {
					int t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wTf[activeOutcomes[aoi]] *= tmp_exp[t][aoi];
				}
				double tmp = 0.0;
				for (int j = 0; j < numOutcomes; j++)
					tmp += wTf[j];

				for (int j = 0; j < numOutcomes; j++)
					S_store[i][j] = wTf[j] / tmp;

				if (!Double.isInfinite(wTf[outcomeList[i]])) {
					Dt -= Math.log(S_store[i][outcomeList[i]]) * numTimesEventsSeen[i];
				}
			}
		}
		return Dt;

	}


	int iter_num;

	public double[][] lbfgs(int maxIterations) {
		//timing
		System.out.println("max iter = " + maxIterations);
		double functionTolerance = 0.000001;
		int M = 0;

		// for convergence test
		Queue<Double> previousVals = new LinkedList<Double>();

		double ro, value;
		double time_end, time_begin, time_pass = 0;

		double[][] newX, x, grad, newGrad, dir;

		S_store = new double[numUniqueEvents][numOutcomes];

		x = new double[numPreds][];
		newX = new double[numPreds][];
		grad = new double[numPreds][];
		newGrad = new double[numPreds][];
		dir = new double[numPreds][];
		for (int t = 0; t < numPreds; t++) {
			int num = params[t].getParameters().length;
			x[t] = new double[num];
			newX[t] = new double[num];
			grad[t] = new double[num];
			newGrad[t] = new double[num];
			dir[t] = new double[num];
		}
		// initialize matrix
		for (int t = 0; t < numPreds; t++)
			for (int i = 0; i < x[t].length; i++)
				x[t][i] = 0;

		// initialize function value
		value = valueAt(x);

		// initialize gradient

		derivativeAt_store(x, grad);

		sList = new ArrayList<double[][]>();
		yList = new ArrayList<double[][]>();
		roList = new ArrayList<Double>();
		double[][] nextS;
		double[][] nextY;
		boolean have_max = (maxIterations > 0);

		begin = System.currentTimeMillis();

		for (iter_num = 0;; iter_num++) {
			time_begin = System.currentTimeMillis();
			double newValue = 0;
			try {
				computeDir(dir, grad);
				if (M > 0 && sList.size() == M || sList.size() == 20) {
					nextS = sList.remove(0);
					nextY = yList.remove(0);
					roList.remove(0);
				} else {
					nextS = new double[x.length][];
					for (int i = 0; i < x.length; i++)
						nextS[i] = new double[x[i].length];
					nextY = new double[x.length][];
					for (int i = 0; i < x.length; i++)
						nextY[i] = new double[x[i].length];
				}

				// perform line search
				newValue = lineSearch(dir, x, newX, grad, value, newGrad);        
			} catch (OutOfMemoryError e) {
				M = sList.size();
				iter_num--;
				continue;
			}

			//derivativeAt_store(newX, newGrad);
			// compute s_k, y_k
			plusAndConstMult(newX, x, -1, nextS);
			plusAndConstMult(newGrad, grad, -1, nextY);
			ro = 1.0 / inner_product(nextS, nextY);
			sList.add(nextS);
			yList.add(nextY);
			roList.add(ro);

			previousVals.add(value);
			int size = previousVals.size();
			double previousVal = size == 10 ? previousVals.remove() : previousVals.peek();
			double averageImprovement = (previousVal - newValue) / size;
			double tmp = 0;
			for (int i = 0; i < x.length; i++)
				for (int j = 0; j < x[i].length; j++)
					tmp += Math.abs(grad[i][j]);

			// output gradient norm
			tmp = 0;

			for (int i = 0; i < x.length; i++)
				for (int j = 0; j < x[i].length; j++)
					tmp += grad[i][j] * grad[i][j];
			tmp = Math.sqrt(tmp);

			if ((tmp < functionTolerance) ||
					(have_max && iter_num >= maxIterations)) {
				clearStuff();
				return newX;
					}
			// shift
			value = newValue;
			double[][] temp = x;
			x = newX;
			newX = temp;

			for (int i = 0; i < x.length; i++)
				for (int j = 0; j < x[i].length; j++)
					grad[i][j] = newGrad[i][j];

			time_end = System.currentTimeMillis();
			time_pass += time_end - time_begin;
			System.out.format(" iter %d time %.4f f %.12g g %.12g ", iter_num + 1, time_pass / 1000.0, value, tmp);
			if (Double.isNaN(value))
				System.exit(1);
			for (int i = 0; i < numPreds; i++)
				for (int  j = 0; j < x[i].length; j++)
					params[i].updateParameter(j, x[i][j] - newX[i][j]);
			MaxentModel m = new GISModel(params, predLabels, outcomeLabels,1,0);
			Testing(m);
		}
	}

	double c1 = 0.01;
	double c2 = 0.9;

	private double lineSearch(double[][] dir, double[][] x, double[][] newX, double[][] grad, double lastValue, double[][] newGrad) {
		double alpha_last = 0;
		double alpha_now = 1;
		double alpha_max = 10;
		int i=1;
		double initial_value = lastValue;
		if ( iter_num <= 2 )
			alpha_now = 0.1;

		double normGradInDir = inner_product(dir, grad);
		while (true)
		{
			double newValue = valueAt((plusAndConstMult(x,dir,alpha_now,newX)));
			if ( (newValue > lastValue + c1*normGradInDir*alpha_now) || ((newValue > lastValue) && i>1))
				return zoom(dir, x, newX, grad, initial_value,normGradInDir,alpha_last, alpha_now, lastValue, newGrad);
			derivativeAt_store(x, newGrad);
			double normGradInDir_new = inner_product(dir, newGrad);
			if ( Math.abs(normGradInDir_new) <= -c2*normGradInDir)
				return newValue;
			if ( normGradInDir_new >= 0 )
				return zoom(dir, x, newX, grad, initial_value,normGradInDir,alpha_now, alpha_last, newValue, newGrad);
			alpha_last = alpha_now;
			alpha_now = (alpha_now+alpha_max)/2.0;
			i++;
		}
	}

	private double zoom(double[][] dir, double[][] x, double[][] newX, double[][] grad, double initial_value,double normGradInDir,double alpha_lo, double alpha_hi, double value_lo, double[][] newGrad) {
		while (true) 
		{
			//interpolate:
			double alphaj = (alpha_lo+alpha_hi)/10.0;
			double newValue = valueAt((plusAndConstMult(x,dir,alphaj,newX)));

			if ( (newValue > initial_value + c1*normGradInDir*alphaj) || (newValue >= value_lo))
				alpha_hi = alphaj;
			else
			{
				derivativeAt_store(x, newGrad);
				double normGradInDir_new = inner_product(dir, newGrad);
				if ( Math.abs(normGradInDir_new) <= -c2*normGradInDir)
					return newValue;
				if (normGradInDir_new*(alpha_hi - alpha_lo) >= 0)
					alpha_hi = alpha_lo;
				alpha_lo = alphaj;
			}
		}
	}  

	private static double dnrm2(double[][] x) {
		double tmp = 0;
		for (int i = 0; i < x.length; i++)
			for (int j = 0; j < x[i].length; j++)
				tmp += x[i][j] * x[i][j];
		return Math.sqrt(tmp);
	}

	private static double ddot(double[][] x, double[][] y) {
		double tmp = 0;
		for (int i = 0; i < x.length; i++)
			for (int j = 0; j < x[i].length; j++)
				tmp += x[i][j] * y[i][j];
		return tmp;
	}

	private static void daxpy(double a, double[][] x, double[][] y) {
		for (int i = 0; i < x.length; i++)
			for (int j = 0; j < x[i].length; j++)
				y[i][j] = a * x[i][j] + y[i][j];
	}

	private static void dscal(double a, double[][] x) {
		for (int i = 0; i < x.length; i++)
			for (int j = 0; j < x[i].length; j++)
				x[i][j] = a * x[i][j];
	}

	private static double norm_inf(double[][] x) {
		double dmax = 0;
		for (int i = 0; i < x.length; i++)
			for (int j = 0; j < x[i].length; j++)
				if (Math.abs(x[i][j]) >= dmax)
					dmax = Math.abs(x[i][j]);
		return dmax;
	}

	public void Hv(double[][] s, double[][] Hs) {
		for (int i = 0; i < Hs.length; i++)
			for (int j = 0; j < Hs[i].length; j++)
				Hs[i][j] = s[i][j] / sigma;
		for (int i = 0; i < numUniqueEvents; i++) {
			double DeltaB = 0.0;
			double Delta [] = new double[numOutcomes];
			for (int j = 0; j < contexts[i].length; j++) {
				int t = contexts[i][j];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					DeltaB += S_store[i][activeOutcomes[aoi]] * s[t][aoi];
					Delta[activeOutcomes[aoi]] += s[t][aoi];
				}
			}

			for (int j = 0; j < contexts[i].length; j++) {
				int t = contexts[i][j];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					Hs[t][aoi] += S_store[i][activeOutcomes[aoi]] * numTimesEventsSeen[i] * (Delta[activeOutcomes[aoi]] - DeltaB);
				}
			}
		}
	}

	private int trcg(double delta, double[][] g, double[][] s,  double[][] r) {
		double[][] d = new double[numPreds][];
		double[][] Hd = new double[numPreds][];
		double rTr, rnewTrnew, alpha, beta, cgtol;

		for (int t = 0; t < g.length; t++) {
			int num = params[t].getParameters().length;
			d[t] = new double[num];
			Hd[t] = new double[num];
		}

		for (int i = 0; i < s.length; i++)
			for (int j = 0; j < s[i].length; j++) {
				s[i][j] = 0;
				r[i][j] = -g[i][j];
				d[i][j] = r[i][j];
			}
		cgtol = 0.2 * dnrm2(r);
		int cg_iter = 0;
		rTr = ddot(r, r);
		while (true) {
			if (dnrm2(r) <= cgtol)
				break;
			cg_iter++;
			Hv(d, Hd);
			alpha = rTr / ddot(d, Hd);
			daxpy(alpha, d, s);
			if (dnrm2(s) > delta) {
				System.out.format("cg reaches trust region boundary%n");
				alpha = -alpha;
				daxpy(alpha, d, s);

				double std = ddot(s, d);
				double sts = ddot(s, s);
				double dtd = ddot(d, d);
				double dsq = delta * delta;
				double rad = Math.sqrt(std * std + dtd * (dsq - sts));
				if (std >= 0)
					alpha = (dsq - sts) / (std + rad);
				else
					alpha = (rad - std) / dtd;
				daxpy(alpha, d, s);
				alpha = -alpha;
				daxpy(alpha, Hd, r);
				break;
			}
			alpha = -alpha;
			daxpy(alpha, Hd, r);
			rnewTrnew = ddot(r, r);
			beta = rnewTrnew / rTr;
			dscal(beta, d);
			daxpy(1.0, r, d);
			rTr = rnewTrnew;
		}
		return cg_iter;
	}

	public double[][] tron(int maxIterations) {
		/* Parameters for updating the iterates. */
		double eta0 = 0.1 , eta1 = 0.25, eta2 = 0.5;
		double sigma1 = 0.1, sigma2 = 0.25, sigma3 = 4;
		int cg_iter, total_cgiter = 0;
		double eps = 1e-14;	
		double delta, snorm;
		double alpha, f, fnew, prered, actred, gs;
		int search = 1, iter = 1, inc = 1, max_iter;
		double time_begin, time_end, time_pass = 0;
		double[][] w = new double[numPreds][];
		double[][] g = new double[numPreds][];
		double[][] s = new double[numPreds][];
		double[][] r = new double[numPreds][];
		double[][] w_new = new double[numPreds][];
		S_store = new double[numUniqueEvents][numOutcomes];
		for (int t = 0; t < numPreds; t++) {
			int num = params[t].getParameters().length;
			w[t] = new double[num];
			g[t] = new double[num];
			s[t] = new double[num];
			r[t] = new double[num];	  
			w_new[t] = new double[num];
		}

		// initialize matrix
		for (int t = 0; t < numPreds; t++)
			for (int i = 0; i < w[t].length; i++)
				w[t][i] = 0;

		f = valueAt(w);
		derivativeAt_store(w, g);
		delta = dnrm2(g);
		System.out.format("initial delta %f%n%n", delta);
		if (norm_inf(g) < eps)
			search = 0;

		iter = 1;
		while (iter <= maxIterations && search == 1) {
			time_begin = System.currentTimeMillis();
			cg_iter = trcg(delta, g, s,  r);
			total_cgiter += cg_iter;

			for (int i = 0; i < w.length; i++)
				for (int j = 0; j < w[i].length; j++)
					w_new[i][j] = w[i][j];
			daxpy(1, s, w_new);

			gs = ddot(g, s);
			prered = -0.5 * (gs - ddot(s, r));
			fnew = valueAt(w_new);

			/* Compute the actual reduction. */
			actred = f - fnew;

			/* On the first iteration, adjust the initial step bound. */
			snorm = dnrm2(s);
			if (iter == 1)
				delta = Math.min(delta, snorm);

			/* Compute prediction alpha*snorm of the step. */

			if (fnew - f - gs <= 0)
				alpha = sigma3;
			else
				alpha = Math.max(sigma1, -0.5 * (gs / (fnew - f - gs)));

			/* Update the trust region bound according to the ratio
			   of actual to predicted reduction. */
			if (actred < eta0*prered)
				delta = Math.min(Math.max(alpha, sigma1) * snorm, sigma2 * delta);
			else if (actred < eta1*prered)
				delta = Math.max(sigma1 * delta, Math.min(alpha * snorm, sigma2 * delta));
			else if (actred < eta2*prered)
				delta = Math.max(sigma1 * delta, Math.min(alpha * snorm, sigma3 * delta));
			else
				delta = Math.max(delta, Math.min(alpha * snorm, sigma3 * delta));

			if (actred > eta0*prered) {
				iter++;
				for (int i = 0; i < s.length; i++)
					for (int j = 0; j < s[i].length; j++)
						w[i][j] = w_new[i][j];
				f = fnew;
				derivativeAt_store(w, g);

				if (norm_inf(g) < eps)
					break;
				time_end = System.currentTimeMillis();
				time_pass += time_end - time_begin;

				double grad_norm = 0;
				for (int i = 0; i < g.length; i++)
					for (int j = 0; j < g[i].length; j++)
						grad_norm += g[i][j] * g[i][j];
				grad_norm = Math.sqrt(grad_norm);
				System.out.format(" iter %d time %.4f f %.12g g %.12g ", iter - 1 , time_pass / 1000.0, f, grad_norm);

				for (int i = 0; i < w.length; i++)
					for (int  j = 0; j < w[i].length; j++)
						params[i].setParameter(j, w[i][j]);
				MaxentModel m = new GISModel(params, predLabels, outcomeLabels,1,0);
				Testing(m);
			}

			if (f < 1.0e-32) {
				System.out.format("warning: f < 1.0e-32%n");
				break;
			}
			if (Math.abs(actred) <= 0 && Math.abs(prered) <= 0) {
				System.out.format("warning: actred and prered <= 0%n");
				break;
			}
			if (Math.abs(actred) <= 1.0e-12*Math.abs(f) &&
					Math.abs(prered) <= 1.0e-12*Math.abs(f)) {
				System.out.format("warning: actred and prered too small%n");
				break;
					}
		}
		System.out.format("%n total CG %d%n", total_cgiter);
		return w;
	}

	public double[][] cddual(int maxIterations) {
		double eps = 1;	
		double Ggap;
		int maxiter = maxIterations;
		int xSize = numUniqueEvents, ySize = numOutcomes, n = numPreds;
		int index[] = new int[xSize];
		double[][] alpha = new double[xSize][ySize];
		double[] P = new double[xSize];
		double[][] w = new double[n][];
		double[][] K = new double[xSize][ySize];
		double[]wTf = new double[ySize];
		double[] g = new double[ySize];
		double[] z = new double[ySize];
		S_store = new double[xSize][ySize];
		double[] alphai;
		double[] Ki;
		double[] wt;
		double gmax=0, gmin=0, gi=0;
		double time_begin, time_end, time_pass = 0, findMinMaxtime, Newtontime;
		int iter = 1, numEvents = 0, pairs, newtons, i, j, t, y;
		double innereps = 1e-5;



		for (t = 0; t < n; t++) {
			int num = params[t].getParameters().length;
			w[t] = new double[num]; // Initial W
			for(int aoi = 0; aoi < num; aoi++)
				w[t][aoi] = 0;
		}
		for (i = 0; i < xSize; i++) {
			P[i] = numTimesEventsSeen[i];
			numEvents += numTimesEventsSeen[i];
		}

		double ratio = Math.pow(10, -2+Math.max( -8, Math.min(-3,-sigma)));
		System.out.format("Ratio %.6g \n", ratio);
		for(i = 0; i < xSize; i++) {
			Ki = K[i];
			alphai = alpha[i];
			for(y = 0; y < ySize; y++) {
				if (y == outcomeList[i]) 
					alphai[y] = P[i] * (1-ratio);
				else 
					alphai[y] = P[i] * ratio / (ySize-1);
				Ki[y] = 0;
			}
		}


		for (i = 0; i < xSize; i++) {
			Ki = K[i];
			alphai = alpha[i];
			for (j = 0; j < contexts[i].length; j++) {
				t = contexts[i][j];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				wt = w[t];
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					Ki[activeOutcomes[aoi]] += 1;
					wt[aoi] -= sigma* alphai[activeOutcomes[aoi]];
					if(activeOutcomes[aoi] == outcomeList[i]) {
						wt[aoi] += sigma * P[i];
					}
				}      
			}
			index[i] = i;
		} 


		Random rgenerator = new Random(10);

		while (iter < maxiter) {
			time_begin = System.currentTimeMillis();  
			findMinMaxtime = 0;
			Newtontime = 0;
			for(i = 0; i < xSize; i++) {
				j = i + rgenerator.nextInt(xSize - i);
				int tmp = index[i];
				index[i] = index[j];
				index[j] = tmp;
			}

			pairs = 0; 
			newtons = 0;
			Ggap = -10;
			for(int s = 0; s < xSize; s++) {
				i = index[s];
				int Maxiter_i = ySize;
				alphai = alpha[i];
				Ki = K[i];

				// Calculate w^T * f  first!
				for (y = 0; y < ySize; y++) {
					wTf[y] = 0;       	  
					z[y] = alphai[y];
				}
				if (isvalue) {              
					for (j = 0; j < contexts[i].length; j++) {
						t = contexts[i][j];
						int[] activeOutcomes = observedExpects[t].getOutcomes();
						wt = w[t];
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
							wTf[activeOutcomes[aoi]] += wt[aoi] * values[i][j];
					}
				}  else {
					for (j = 0; j < contexts[i].length; j++) {
						t = contexts[i][j];
						int[] activeOutcomes = observedExpects[t].getOutcomes();
						wt = w[t];
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++) 
							wTf[activeOutcomes[aoi]] += wt[aoi];
					}
				} 

				gmax = - Double.MAX_VALUE;
				gmin = Double.MAX_VALUE;
				int y1_ = 0, y2_ = 0;
				double tmpg;
				for(y = 0; y < ySize; y++) {
					tmpg = Math.log(z[y]) + 1 - wTf[y];
					if(tmpg >= gmax) {
						y1_ = y;
						gmax = tmpg;
					}
					if (tmpg < gmin) {
						y2_ = y;
						gmin = tmpg;
					}
					g[y] = tmpg;
				}
				Ggap = Math.max(gmax-gmin, Ggap);

				eps = Math.abs((gmax-gmin) * 0.7);

				int iter_i = 0, niter = 0;
				boolean touchbound = false;
				while (iter_i < Maxiter_i) {
					pairs++;
					if(iter_i > 0) {
						y1_ = y2_ = 0;
						gmin = gmax = g[0];
						for(y = 1; y < ySize; y++) {
							if(g[y] >= gmax) {
								gmax = g[y];
								y1_ = y;
							}
							if(g[y] < gmin) {
								gmin = g[y];
								y2_ = y;
							}
						}
					}
					if(Math.abs(gmax-gmin)< eps) {
						break;
					}
					int y1 = y1_, y2 = y2_;
					double range = z[y1]+z[y2];
					double a = sigma * (Ki[y1]+Ki[y2]);
					double b = sigma * ((z[y1]-alphai[y1]) * Ki[y1] - (z[y2]-alphai[y2])*Ki[y2])+ wTf[y2] - wTf[y1];
					double Dp, Dpp, d = 0, tmpd;
					// Enforce  d* <= dm
					if((z[y2]-z[y1])*0.5*a+b < 0) {
						y1 = y2_;
						y2 = y1_;
						b = -b;
					}
					double x = z[y1]; //Math.min(z[y1], 0.1*range);
					double c1 = z[y1], c2 = z[y2];
					double tmpx;
					if(range-x < 0.5*range) x = 0.1*range;


					niter = 0;

					while(true) {
						newtons++;
						Dp = a*(x-c1) + b + Math.log(x/(range-x));
						if(Math.abs(Dp) < eps*0.5 /*innereps*/) 
							break;
						Dpp = a + range/(range-x)/x;
						tmpx = x - Dp/Dpp;
						if( tmpx <= 0) {
							x *= 0.1;
						} else if (tmpx >= range) {
							x += 0.9*(range-x);
							System.out.format("should not be here\n");
						} else {
							x = tmpx;
						}
						if(x < 1e-100) {
							touchbound = true;
							break;
						}
						niter++;
						if(niter > 50) break;
					}
					z[y1] = x;
					z[y2] = range-x;

					g[y1] = Math.log(x) + 1 - wTf[y1] + sigma * (x-alphai[y1]) * Ki[y1];
					g[y2] = Math.log(range-x) + 1 - wTf[y2] + sigma * (range-x-alphai[y2]) * Ki[y2];
					iter_i++;
				}

				// Update w(\AL_i + z) = w(\AL_i) - sigma * \sum_y z[y] f(x_i, y)
				for (j = 0; j < contexts[i].length; j++) {
					t = contexts[i][j];
					int[] activeOutcomes = observedExpects[t].getOutcomes();
					wt = w[t];
					for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
						wt[aoi] -= sigma * (z[activeOutcomes[aoi]] - alphai[activeOutcomes[aoi]]);
				} 
				// Update alpha_i
				for(y = 0; y < ySize; y++)
					alphai[y] = z[y];
			}

			time_end = System.currentTimeMillis();
			time_pass += time_end - time_begin;

			double L=0;
			for (t = 0; t < n; t++) {
				for (int aoi = 0; aoi < w[t].length; aoi++)
					L += w[t][aoi] * w[t][aoi];  	
			}
			L /= 2*sigma; 

			for (i = 0; i < xSize; i++) {
				for (y =0; y < ySize; y++) {
					if (alpha[i][y]>0)
						L += alpha[i][y] * Math.log(alpha[i][y]);
				}  
				L -= P[i] * Math.log(P[i]);
			}   

			double f;
			double grad_norm = 0;	  
			double w_norm = 0;
			double[][] grad = new double[n][];
			for (t = 0; t < n; t++) {
				grad[t] = new double[w[t].length];
			}

			for (int ii = 0; ii < w.length; ii++)
			  for (int  jj = 0; jj < w[ii].length; jj++)
			  {
				params[ii].setParameter(jj, w[ii][jj]);
				w_norm += w[ii][jj]*w[ii][jj];
			  }
				
			f = valueAt(w);
			derivativeAt_store(w, grad);
			
			for (t = 0; t < n; t++) {
				double[] gradt = grad[t];
				for (int aoi = 0; aoi < grad[t].length; aoi++) {
					grad_norm += gradt[aoi] * gradt[aoi]; 
				}
			}
			grad_norm = Math.sqrt(grad_norm);

			double grad_norm2 = 0;

			System.out.format(" iter %d time %.4f f %.12g g %.12g dual_f %.12g gap %g ", iter , (time_pass) / 1000.0, f, grad_norm, L, Ggap);

			MaxentModel m = new GISModel(params, predLabels, outcomeLabels, 1, 0);
			Testing(m);    

			iter++;     
		}  
		return w;
	}

	public double[][] EG(int maxIterations) {
		double eps = 1e-0;	
		double gap = 1;
		int maxiter = maxIterations;
		int xSize = numUniqueEvents, ySize = numOutcomes, n = numPreds;
		int index[] = new int[xSize];
		double[][] alpha = new double[xSize][ySize];
		double[] P = new double[xSize];
		double[][] w = new double[n][];
		double[][] K = new double[xSize][ySize];
		double[] wTf = new double[ySize];
		double[] expwtf = new double[ySize];
		double[] z = new double[ySize];
		double[] lrates = new double[xSize];
		double[] alphai;
		double[] Ki;
		double[] wt;
		S_store = new double[xSize][ySize];
		double max_gi=0, min_gi=0, gi=0;
		double time_begin, time_end, time_pass = 0;
		int iter = 1, numEvents = 0, i,j,t,y;
		double innereps = 1e-5;
		double lrate = 0.5;


		for (t = 0; t < n; t++) {
			int num = params[t].getParameters().length;
			w[t] = new double[num]; // Initial W
		}
		for (i = 0; i < xSize; i++) {
			P[i] = numTimesEventsSeen[i];
			numEvents += numTimesEventsSeen[i];
		}

		double ratio = Math.pow(10, -2+Math.max( -8, Math.min(-3,-sigma)));
		System.out.format("Ratio %.6g \n", ratio);
		for(i = 0; i < xSize; i++) {
			Ki = K[i];
			alphai = alpha[i];
			for(y = 0; y < ySize; y++) {
				if (y == outcomeList[i]) alphai[y] = P[i] * (1-ratio);
				else alphai[y] = P[i] * ratio / (ySize-1);
				Ki[y] = 0;
			}
			lrates[i] = 0.5;
		}


		for (i = 0; i < xSize; i++) {
			Ki = K[i];
			alphai = alpha[i];
			for (j = 0; j < contexts[i].length; j++) {
				t = contexts[i][j];
				int[] activeOutcomes = observedExpects[t].getOutcomes();
				wt = w[t];
				for (int aoi = 0; aoi < activeOutcomes.length; aoi++) {
					Ki[activeOutcomes[aoi]] += 1;
					wt[aoi] -= sigma* alphai[activeOutcomes[aoi]];
					if(activeOutcomes[aoi] == outcomeList[i]) {
						wt[aoi] += sigma * P[i];
					}
				}      
			}
			index[i] = i;
		} 



		Random rgenerator = new Random(10);

		while (iter < maxiter) {
			time_begin = System.currentTimeMillis();  
			int fails = 0;
			/*
			   for(i = 0; i < xSize; i++) {
			   j = i + rgenerator.nextInt(xSize - i);
			   int tmp = index[i];
			   index[i] = index[j];
			   index[j] = tmp;
			   }
			*/
			for(int s = 0; s < xSize; s++) {
				//i = index[s];
				i = rgenerator.nextInt(xSize);
				int Maxiter_i = ySize;
				alphai = alpha[i];
				Ki = K[i];
				double olddual = 0;

				// Calculate w^T * f  first!
				for (y = 0; y < ySize; y++) {
					wTf[y] = 0;       	  
					z[y] = alphai[y];
					olddual += alphai[y] * Math.log(alphai[y]);
				}
				if (isvalue) {              
					for (j = 0; j < contexts[i].length; j++) {
						t = contexts[i][j];
						int[] activeOutcomes = observedExpects[t].getOutcomes();
						wt = w[t];
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
							wTf[activeOutcomes[aoi]] += wt[aoi] * values[i][j];
					}
				}  else {
					for (j = 0; j < contexts[i].length; j++) {
						t = contexts[i][j];
						int[] activeOutcomes = observedExpects[t].getOutcomes();
						wt = w[t];
						for (int aoi = 0; aoi < activeOutcomes.length; aoi++) 
							wTf[activeOutcomes[aoi]] += wt[aoi];
					}
				} 
				for(y = 0; y < ySize; y++) 
					expwtf[y] = Math.exp(1-wTf[y]);
				
				int LRATE_MAXCUTS = 8;
				int num_remaining_cuts = (iter == 1)? 2 : LRATE_MAXCUTS;
				while (true) {
					double sumZ = 0, dual_dec = olddual;
					for(y = 0; y < ySize; y++) {
						z[y] = Math.max(1e-100,alphai[y]*Math.pow(alphai[y]*expwtf[y],-lrates[i]));
						sumZ += z[y];
					}
					sumZ = sumZ/P[i];
					for(y = 0; y < ySize; y++) {
						z[y] /= sumZ;
						double z_diff = z[y] - alphai[y];
						dual_dec -= z[y]*Math.log(z[y]) - z_diff*wTf[y] + 0.5*sigma*z_diff*z_diff*Ki[y];
					}
					if(dual_dec < 0) {
						if(num_remaining_cuts > 0) {
							num_remaining_cuts--;
							lrates[i] *= 0.5;
						} else {
							fails++;
							break;
						}
					}else {
						lrates[i] *= 1.05;
						// Update w(\AL_i + z) = w(\AL_i) - sigma \sum_y z[y] f(x_i, y)
						for (j = 0; j < contexts[i].length; j++) {
							t = contexts[i][j];
							int[] activeOutcomes = observedExpects[t].getOutcomes();
							wt = w[t];
							for (int aoi = 0; aoi < activeOutcomes.length; aoi++)
								wt[aoi] -= sigma * (z[activeOutcomes[aoi]] - alphai[activeOutcomes[aoi]]);
						} 
						// Update alpha_i
						for(y = 0; y < ySize; y++)
							alphai[y] = z[y];
						break;
					} 
				}
			}


			time_end = System.currentTimeMillis();
			time_pass += time_end - time_begin;

			double L=0;
			for (t = 0; t < n; t++) {
				for (int aoi = 0; aoi < w[t].length; aoi++)
					L += w[t][aoi] * w[t][aoi];  	
			}
			L /= 2*sigma; 

			for (i = 0; i < xSize; i++) {
				for (y =0; y < ySize; y++) {
					if (alpha[i][y]>0)
						L += alpha[i][y] * Math.log(alpha[i][y]);
				}  
				L -= P[i] * Math.log(P[i]);
			}   


			double f;
			double grad_norm = 0;	  
			double[][] grad = new double[n][];
			for (t = 0; t < n; t++) {
				grad[t] = new double[w[t].length];
			}

			for (int ii = 0; ii < w.length; ii++)
			  for (int  jj = 0; jj < w[ii].length; jj++)
				params[ii].setParameter(jj, w[ii][jj]);

			f = valueAt(w);
			derivativeAt_store(w, grad);
			for (t = 0; t < n; t++) {
				double[] gradt = grad[t];
				for (int aoi = 0; aoi < grad[t].length; aoi++) {
					grad_norm += gradt[aoi] * gradt[aoi]; 
				}
			}
			grad_norm = Math.sqrt(grad_norm);

			System.out.format(" iter %d time %.4f f %.12g g %.12g dual_f %.12g  ", iter , (time_pass) / 1000.0, f, grad_norm, L);

			MaxentModel m = new GISModel(params, predLabels, outcomeLabels, 1, 0);
			Testing(m);    

			iter++;     
		}  
		return w;  
	} 	
}

